/*
 * Threshold-RRT.hpp
 *
 *  Created on: Feb 05, 2013
 *      Author: ddevaurs
 */

#ifndef _THRESHOLD_RRT_HPP_
#define _THRESHOLD_RRT_HPP_

#include "planner/Diffusion/RRT.hpp"


/**
 * Expansion procedure of the obstacleness-aware RRT (RRT_obst) algorithm,
 * which involves an adaptive cost threshold.
 * The threshold value can be defined by the user in the GUI (if left at 0, it means 'not user-defined').
 * If no value is defined by the user, the threshold value is automatically set to be
 * the minimal cost of all the nodes present in the graph at initialization time.
 */
class ThresholdExpansion: public RRTExpansion
{
public:
    //! Constructor.
	ThresholdExpansion(Graph * graph, Node * goal);

    //! Destructor.
    virtual ~ThresholdExpansion() {};

    /**
     * Attempt to link two given nodes (i.e. two components, when relevant).
     * @return TRUE if the junction is successful.
     */
    bool attemptJunction(Node * fromNode, Node * toNode);

private:
    /**
     * RRT_obst Extend.
     * @param fromNode: node from which the expansion is performed
     * @param directionConf: configuration toward which the expansion is going
     * @param directionNode: node toward which the expansion is going (when relevant)
     * @return the number of created nodes
     */
    unsigned extend(Node * fromNode, confPtr_t directionConf, Node * directionNode);

    /**
     * RRT_obst Connect.
     * @param fromNode: node from which the expansion is performed
     * @param directionConf: configuration toward which the expansion is going
     * @param directionNode: node toward which the expansion is going (when relevant)
     * @return the number of created nodes
     */
    unsigned connect(Node * fromNode, confPtr_t directionConf, Node * directionNode);

private:
	//! value of the cost threshold
	double threshold;

	//! increment for the cost threshold
	double increment;
};


/**
 * The obstacleness-aware RRT (RRT_obst) algorithm, which involves an adaptive cost threshold.
 */
class ThresholdRRT: public RRT
{
public:
    //! Constructor.
	ThresholdRRT(Robot * robot, Graph * graph) : RRT(robot, graph) {
        std::cout << "Construct the Threshold-RRT" << std::endl;
    }

    //! Destructor.
    virtual ~ThresholdRRT() {};

    /**
     * Initialize the RRT_obst.
     * @return the number of nodes added to the graph
     */
    unsigned init();
};

#endif
